import org.eclipse.swt.widgets.*;

public class AlertUSB {
    public final static String VERSION = "20140830";
    public final static int DEV_USB = 1<<0;
    public final static int DEV_DVD = 1<<1;
    
    public Display display;
    public Shell shell;
    public Shell dialogShell;
    public Preference preference;
    public Net net;

    public AlertUSB(Display display) {
        this.display = display;
        this.shell = new Shell(display);
        this.dialogShell = new Shell(shell);
        preference = new Preference(this, shell);
        preference.load();
        net = new Net(preference);
    }

    public static void main(String[] args) {
        AlertUSB alertUSB = new AlertUSB(new Display());
        if (args.length == 0) {
        	alertUSB.config();
        } else {
            int device = alertUSB.getExistDev(args);
            alertUSB.alert(device);
        }
        System.out.println("finished");
    }

    /** 環境設定 */
    public void config() {
        preference.open();
    }
    
    /** 検査する */
    public int getExistDev(String args[]) {
        int device = 0;
        for (String arg : args) {
            arg = arg.toUpperCase();
            if (arg.equals("/CHECK")) {
                if (preference.watchUSB() && existUSB()) device ^= DEV_USB;
                if (preference.watchDVD() && existDVD()) device ^= DEV_DVD;
            }
            else if (arg.equals("/USB")) device ^= DEV_USB;
            else if (arg.equals("/DVD")) device ^= DEV_DVD;
            else System.out.println("Unknown option : " + arg);
        }
        return device;
    }
    
    public boolean existUSB() {
        String drive = new DeviceList().getAllDevices(DEV_USB);
        return drive.length() > 0;
    }
 
    public boolean existDVD() {
        String drive = new DeviceList().getAllDevices(DEV_DVD);
        return drive.length() > 0;
    }
    
    /** 警告動作 */
    public void alert(int device) {
        if (device == 0) return;
        alertConsole(device);
        if (preference.noticeDialog()) alertDialog(device);
        if (preference.noticeWave())   alertWave(device);
        if (preference.noticeBeep())   alertBeep(device);
        if (preference.noticeHard())   alertHard(device);
        if (preference.noticeNet())    alertNet(device);
        loop();
    }
    
    public void alertConsole(int device) {
        if ((device & DEV_USB) != 0) System.out.println("USB!!!");
        if ((device & DEV_DVD) != 0) System.out.println("DVD!!!");
    }
    
    public void alertDialog(int device) {
        new MyDialog(dialogShell, device).start();
    }
    
    public void alertWave(int device) {
        int len = preference.noticeSoundLen();
        int cyc = preference.noticeSoundCyc();
        new Wave(len/cyc, cyc).start();
    }
    
    public void alertBeep(int device) {
        int len = preference.noticeSoundLen();
        int cyc = preference.noticeSoundCyc();
        new Beep(len/cyc, cyc).start();
    }
    
    public void alertHard(int device) {
        int len = preference.noticeSoundLen();
        int cyc = preference.noticeSoundCyc();
        new Hard(len/cyc, cyc).start();
    }
    
    public void alertNet(int device) {
        net.start();
    }

    /** ウインドウが閉じられるまでループ */
    public void loop() {
        while (dialogShell.isVisible()) {
            if (!display.readAndDispatch()) display.sleep();
        }
        display.dispose();
    }
}
