import java.io.*;
import org.eclipse.jface.preference.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;

/**
 * @author tutimura
 *
 */
public class Preference extends Thread {
    public AlertUSB alertUSB;

    private int noticeSoundLen;
    private int noticeSoundCyc;
    private StringFieldEditor netAddr;  
    private Text netAddrExtract;  

    private Group gDialog, gWave, gBeep, gHard;
    
    public class NoticeSoundPage extends FieldEditorPreferencePage {
        private NoticeSoundPage() {
                super("AlertUSB - 一般通知", GRID);
        }
        private Group createGroup(String text, String psname,
                SelectionListener sl) {
            Group g = new Group(getFieldEditorParent(), SWT.SHADOW_IN);
            g.setText(text);

            addField(new BooleanFieldEditor(psname, "通知する", g));

            Button b = new Button(g, SWT.PUSH);
            b.setText("テスト");
            b.addSelectionListener(sl);

            new Label(g, SWT.NONE);

            GridData data = new GridData(GridData.FILL_HORIZONTAL);
            g.setLayoutData(data);
            return g;
        }
        
        protected void createFieldEditors() {
            gDialog = createGroup("ダイヤログ", "notice.dialog", new SelectionAdapter() {
                public void widgetSelected(SelectionEvent e) {
                    alertUSB.alertDialog(AlertUSB.DEV_USB);
                }
            });
            
            gWave = createGroup("Waveサウンド", "notice.wave", new SelectionAdapter() {
                public void widgetSelected(SelectionEvent e) {
                    alertUSB.alertWave(AlertUSB.DEV_USB);
                }
            });

            gBeep = createGroup("BEEP回路", "notice.beep", new SelectionAdapter() {
                public void widgetSelected(SelectionEvent e) {
                    alertUSB.alertBeep(AlertUSB.DEV_USB);
                }
            });
            {
                Label l = (Label) gBeep.getChildren()[2];
                if (new Beep().isReady()) {
                    l.setText("ドライバあり");
                } else {
                    l.setText("ドライバなし");
                    gBeep.getChildren()[1].setEnabled(false);
                }
            }

            gHard = createGroup("専用ハード", "notice.hard", new SelectionAdapter() {
                public void widgetSelected(SelectionEvent e) {
                    alertUSB.alertHard(AlertUSB.DEV_USB);
                }
            });
            {
                Label l = (Label) gHard.getChildren()[2];
                if (new Hard().isReady()) {
                    l.setText("ハード接続");
                } else {
                    l.setText("ハードなし");
                }
            }

            Group g = new Group(getFieldEditorParent(), SWT.SHADOW_IN);
            g.setText("サウンド共通");
            GridData data = new GridData(GridData.FILL_HORIZONTAL);
            data.horizontalSpan = 4;
            g.setLayoutData(data);

            final ScaleFieldEditor lenEditor = new ScaleFieldEditor(
                    "notice.sound.len",
                    "発音時間" + noticeSoundLen + "(ms)",
                    g, 0, 5000, 50, 500);
            final Scale lenScale = lenEditor.getScaleControl();
            lenScale.addSelectionListener(new SelectionAdapter(){
                public void widgetSelected(SelectionEvent e){
                    noticeSoundLen = lenScale.getSelection();
                    lenEditor.setLabelText("発音時間" + noticeSoundLen + "(ms)");
                }
            });
            addField(lenEditor);

            final ScaleFieldEditor cycEditor = new ScaleFieldEditor(
                    "notice.sound.cyc",
                    "断続周期" + noticeSoundCyc + "(ms)",
                    g, 10, 1000, 5, 100);
            final Scale cycScale = cycEditor.getScaleControl();
            cycScale.addSelectionListener(new SelectionAdapter(){
                public void widgetSelected(SelectionEvent e){
                    noticeSoundCyc = cycScale.getSelection();
                    cycEditor.setLabelText("断続周期" + noticeSoundCyc + "(ms)");
                }
            });
            addField(cycEditor);
        }
        @Override
        protected Control createContents(Composite parent) {
            Composite c = (Composite) super.createContents(parent);
            GridLayout layout = new GridLayout();
            layout.numColumns = 4;
            c.setLayout(layout);
            return c;
        }
    }

    public class NoticeNetPage extends FieldEditorPreferencePage {
        private NoticeNetPage() {
                super("AlertUSB - ネット通知", GRID);
        }
        protected void createFieldEditors() {
            addField(new BooleanFieldEditor("notice.net",
                    "ネットワーク", getFieldEditorParent()));
//            addField(new ComboFieldEditor("notice.net.protocol",
//                    "プロトコル", new String[][] {
//                    {"http", "http"},
//                    {"https", "https"},
//            }, getFieldEditorParent()));

            addField(netAddr = new StringFieldEditor("notice.net.addr",
                    "アドレス", getFieldEditorParent()));
            Button b = new Button(getFieldEditorParent(), SWT.PUSH);
            b.setText("テスト");
            b.addSelectionListener(new SelectionAdapter() {
                public void widgetSelected(SelectionEvent e) {
                    alertUSB.alertNet(AlertUSB.DEV_USB);
                }
            });
            netAddrExtract = new Text(getFieldEditorParent(), SWT.BORDER);
            netAddrExtract.setEditable(false);
            Text t = netAddr.getTextControl(getFieldEditorParent());
            t.addModifyListener(new ModifyListener() {
                public void modifyText(ModifyEvent e) {
                    String s = alertUSB.net.getURL(netAddr.getStringValue());
                    netAddrExtract.setText(s);
                    netAddrExtract.pack();
                }
            });
        }
    }

    public class WatchPage extends FieldEditorPreferencePage {
        private WatchPage() {
                super("AlertUSB - 監視デバイス", GRID);
        }
        protected void createFieldEditors() {
            addField(new BooleanFieldEditor("watch.usb",
                    "USBメモリ", getFieldEditorParent()));
            addField(new BooleanFieldEditor("watch.dvd",
                    "CD/DVDドライブ", getFieldEditorParent()));
        }
    }

    public class AboutPage extends FieldEditorPreferencePage {
        private AboutPage() {
                super("AlertUSB - about", GRID);
        }
        protected void createFieldEditors() {
            Text t = new Text(getFieldEditorParent(), SWT.MULTI);
            t.setEditable(false);
            t.setText(
                    "Version " + AlertUSB.VERSION + "\n" +
                    "http://ist.ksc.kwansei.ac.jp/~tutimura/AlertUSB/\n" +
                    "\n" +
                    "(C) Noyubuki Tsuchimura, \n" + 
                    "Phillip Gibbons (inpout32.dll), \n" +
                    "Xavier Clerc (wave sound BEEP), \n" +
                    "KSL Embedded (USB access), \n" +
                    "Timothy Wall (JNA), \n" +
                    "Eclipse Foundation Software (SWT)" +
            "");
            t.pack();
        }
    }

    private PreferenceStore ps;
    private boolean isUnix;
    private Shell shell;
    private PreferenceDialog dialog;
    
    public Preference(AlertUSB alertUSB, Shell shell) {
        this.alertUSB = alertUSB;
        this.shell = shell;

        String path = System.getProperty("java.class.path");
        if (!path.contains(File.pathSeparator)
                && path.endsWith(".jar")) { // jar による起動
            // jar と同じフォルダの rc
            path = path.replaceFirst("jar$", "rc");
        } else {
            path = "AlertUSB.rc";  // カレントの rc
        }
        System.out.println("config file : " + path);
        ps = new PreferenceStore(path);
        ps.setDefault("notice.dialog", true);
        ps.setDefault("notice.wave", true);
        ps.setDefault("notice.beep", true);
        ps.setDefault("notice.sound.len", 2500);
        ps.setDefault("notice.sound.cyc", 100);
        ps.setDefault("notice.net.protocol", "http://");
        ps.setDefault("watch.usb", true);
        isUnix = File.separatorChar == '/';
        if (isUnix) setDefaultUnix();
        else        setDefaultWin();
    }
   
    /** 初期値の上書き(UNIX) */
    private void setDefaultUnix() {
        ps.setDefault("notice.beep", false);
    }
    
    /** 初期値の上書き(Windows) */
    private void setDefaultWin() {
    }
    
    public boolean isUnix()       { return isUnix; }

    public boolean noticeDialog() { return ps.getBoolean("notice.dialog"); }
    public boolean noticeWave()   { return ps.getBoolean("notice.wave"); }
    public boolean noticeBeep()   { return ps.getBoolean("notice.beep"); }
    public boolean noticeHard()   { return ps.getBoolean("notice.hard"); }
    public int noticeSoundLen()   { return noticeSoundLen; }
    public int noticeSoundCyc()   { return noticeSoundCyc; }
    public boolean noticeNet()    { return ps.getBoolean("notice.net"); }
    public String noticeNetProtocol() { return ps.getString("notice.net.protocol"); }
    public String noticeNetAddr()  { return ps.getString("notice.net.addr"); }
    
    public boolean watchUSB()     { return ps.getBoolean("watch.usb"); }
    public boolean watchDVD()     { return ps.getBoolean("watch.dvd"); }

    public boolean load() {
        boolean ret = true;
        try {
            ps.load();
        } catch (FileNotFoundException e) {
            ret = false;
        } catch (IOException e) {
            e.printStackTrace();
            ret = false;
        }
        noticeSoundLen = ps.getInt("notice.sound.len");
        noticeSoundCyc = ps.getInt("notice.sound.cyc");
        return ret;
    }
    
    public void open() {
        PreferenceManager manager = new PreferenceManager();

        PreferenceNode noticeSoundNode = new PreferenceNode("notice-sound", new NoticeSoundPage());
        manager.addToRoot(noticeSoundNode);
        
        PreferenceNode noticeNetNode = new PreferenceNode("notice-net", new NoticeNetPage());
        manager.addToRoot(noticeNetNode);
        
        PreferenceNode watchNode = new PreferenceNode("watch", new WatchPage());
        manager.addToRoot(watchNode);

        PreferenceNode aboutNode = new PreferenceNode("about", new AboutPage());
        manager.addToRoot(aboutNode);

        dialog = new PreferenceDialog(shell, manager);
        dialog.setPreferenceStore(ps);
        //shell.getDisplay().asyncExec(new Update()); // call run()
        dialog.open(); // && wait
    }

    class Update extends Thread {
        public void run() {
            boolean hard = true;
            
            while (!shell.isDisposed()) {
                System.out.println("running...");
                Label lHard = (Label) gHard.getChildren()[2];
                if (!isUnix() && new Hard().isReady()) {
                    if (!hard) {
                        lHard.setText("ハード接続");
                        gHard.getChildren()[1].setEnabled(true);
                        gHard.redraw();
                        hard = true;
                    }
                } else {
                    if (hard) {
                        lHard.setText("ハードなし");
                        gHard.getChildren()[1].setEnabled(false);
                        gHard.redraw();
                        hard = false;
                    }
                }
                
                try {
                    sleep(500);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
                //if (!shell.isDisposed()) dialog.updateButtons();
            }
            System.out.println("running... stopped");
        }
    }
}
